/* polygon.h: definitions for polygon package */

#ifndef POLYGON_H
#define POLYGON_H

#define POLYGON_MAX_VERTICES 10  /* max #sides to a polygon; change if needed */
/*
 * Note that poly_clip, given an n-gon as input, might output an (n+6)gon.
 * POLYGON_MAX_VERTICES=10 is thus appropriate if input polygons are triangles
 * or quads.
 */

typedef struct
{                               /* A POLYGON VERTEX */
   /* world space position */
   double x;
   double y;
   double z;

   /* screen space position (sometimes homo.) */
   double sx;
   double sy;
   double sz;
   double sw;

   /* (red, green, blue) color */
   double r;
   double g;
   double b;

   /* world space normal vector */
   double nx;
   double ny;
   double nz;

   /* texture position (sometimes homogeneous) */
   double u;
   double v;
   double q;
}               vertex;

/* update poly.c if you change this structure */
/* note: don't put > 32 doubles in Poly_vert struct, or mask will overflow */

typedef struct
{                              /* A POLYGON */
#if 0
   int            max_n;
#endif
   int            n;           /* number of active sides */
   unsigned long  mask;        /* interpolation mask for vertex elems */
   vertex        *Vertices;    /* vertices */
} polygon;

/*
 * mask is an interpolation mask whose kth bit indicates whether the kth
 * double in a Poly_vert is relevant.
 * For example, if the valid attributes are sx, sy, and sz, then set
 *      mask = POLY_MASK(sx) | POLY_MASK(sy) | POLY_MASK(sz);
 */

typedef struct
{                               /* A BOX (TYPICALLY IN SCREEN SPACE) */
   /* left and right */
   double x0;
   double x1;

   /* top and bottom */
   double y0;
   double y1;

   /* near and far */
   double z0;
   double z1;
} view_box;

typedef struct
{                               /* WINDOW: A DISCRETE 2-D RECTANGLE */
   /* xmin and ymin */
   int LL_x;
   int LL_y;

   /* xmax and ymax (inclusive) */
   int UR_x;
   int UR_y;
} window;

#define VERTEX_MASK(elem) \
        (1 << (&(Vertex_dummy->elem) - (double *)Vertex_dummy))

#define POLYGON_CLIP_OUT     0     /* polygon entirely outside box */
#define POLYGON_CLIP_PARTIAL 1     /* polygon partially inside */
#define POLYGON_CLIP_IN      2     /* polygon entirely inside box */

/* used superficially by POLY_MASK macro */

#ifdef POLYGON_C
vertex *Vertex_dummy;
#else
extern vertex *Vertex_dummy;
#endif

#endif

polygon *new_n_polygons(int Num_polygons, int Num_vertices);
void     free_polygons(polygon *Polygons);


void            polygon_print(char *str, polygon *p);
void            polygon_vertex_label(char *str, unsigned long mask);
void            polygon_vertex_print(char *str, vertex *v, unsigned long mask);
int             polygon_clip_to_box( /* p1, box */ );
void            polygon_clip_to_halfspace( /* p, q, index, sign, k, name */ );
void            polygon_scan_convert( /* p, win, pixelproc */ );
